@@ -22,8 +22,8 @@ from .configobj import ConfigObject
 from .exception import GitProjectException
 from .git import Git
 from .gitproject import GitProject
-from .item import ConfigObjectItem
-from .parsermanager import gen_config_epilog, gen_runnable_epilog, ParserManager
+from .main import main_impl
+from .parsermanager import ParserManager
 from .plugin import Plugin
 from .pluginmanager import PluginManager
 from .project import Project
