@@ -121,8 +121,13 @@ class Git(object):
                         raise GitProjectException('Single value get for multival')
                     return next(iter(self._values))
 
+                def has_value(self, value):
+                    """Return whether the given value is in the item."""
+                    return value in self._values
+
                 def add_value(self, value):
                     """Add a value to this key, potentially turning it into a multi-value key."""
+                    assert not value in self._values
                     self._values.add(value)
 
                 def clear(self):
@@ -140,10 +145,15 @@ class Git(object):
                     self._values = {value for value in self._values
                                     if not re.search(pattern, value)}
 
-            def __init__(self, git, name):
+            def __init__(self, git, name, content_dict):
                 self._git = git
                 self._name = name
                 self._items = dict()
+                for key, values in content_dict.items():
+                    config_item = self.ConfigItem(self.itemname(key))
+                    for value in values:
+                        config_item.add_value(value)
+                    self._items[self.itemname(key)] = config_item
 
             @property
             def name(self):
@@ -176,6 +186,9 @@ class Git(object):
                 """Add a value to the given key, potentially turning it into a multi-value key.
 
                 """
+                if self.has_value(key, value):
+                    return
+
                 self._git._repo.config.set_multivar(self.itemname(key), re.escape(value), value)
                 item = self._items.get(self.itemname(key), None)
                 if not item:
@@ -187,6 +200,13 @@ class Git(object):
                 """Return whether this key exists in this section."""
                 return True if self._items.get(self.itemname(key), None) else False
 
+            def has_value(self, key, value):
+                """Return whether this key with this values exists in this section."""
+                item = self._items.get(self.itemname(key), None)
+                if not item:
+                    return False
+                return item.has_value(value)
+
             def get_item(self, key):
                 """Get the key:value pair for this key."""
                 item = self._items.get(self.itemname(key), None)
@@ -235,13 +255,29 @@ class Git(object):
             self._sections = dict()
 
             if self._git.has_repo():
+                # First gather the contents of each section.  We do it this way
+                # because iteratively adding key, value means will will call
+                # add_entry for each item.  Since the underlying config already
+                # contains everything, we don't want to call add_entry again and
+                # add duplicates.  So we call a proper ConfigSection
+                # intialization after gathering all the entries.
+                sections = dict()
                 for entry in config:
                     section, key = entry.name.rsplit('.', 1)
-                    section_entry = self.get_section(section)
-                    if not section_entry:
-                        section_entry = self.ConfigSection(self._git, section)
-                    section_entry.add_item(key, entry.value)
-                    self._sections[section] = section_entry
+                    if not section in sections:
+                        sections[section] = dict()
+                    section_entry = sections[section]
+                    if not key in section_entry:
+                        section_entry[key] = set()
+                    section_entry[key].add(entry.value)
+
+                self._git.validate_config()
+                for section_name, section_dict in sections.items():
+                    config_section = self.ConfigSection(self._git,
+                                                        section_name,
+                                                        section_dict)
+                    self._sections[section_name] = config_section
+                self._git.validate_config()
 
         def get_section(self, section):
             """Get the named section from the config."""
@@ -258,7 +294,7 @@ class Git(object):
             """Set the value of key under section named by section_name to value."""
             section = self._sections.get(section_name, None)
             if not section:
-                section = self.ConfigSection(self._git, section_name)
+                section = self.ConfigSection(self._git, section_name, dict())
             section.set_item(key, value)
             self._sections[section_name] = section
 
@@ -269,8 +305,9 @@ class Git(object):
             """
             section = self._sections.get(section_name, None)
             if not section:
-                section = self.ConfigSection(self._git, section_name)
-            section.add_item(key, value)
+                section = self.ConfigSection(self._git, section_name, dict())
+            if not section.has_value(key, value):
+                section.add_item(key, value)
             self._sections[section_name] = section
 
         def iter_multival(self, section_name, key):
@@ -346,6 +383,7 @@ class Git(object):
         if repo_path:
             self._repo = pygit2.Repository(repo_path)
             self._config = self.Config(self, self._repo.config)
+            self.validate_config()
 
     @property
     def config(self):
@@ -693,3 +731,57 @@ class Git(object):
         refspecs = [f':{refname}']
         remote = self._repo.remotes[remote]
         remote.push(refspecs, callback)
+
+    def validate_config(self):
+        """Ensure the git config is sane.  This is primarily a development debugging
+        tool."""
+        if not self.has_repo():
+            return
+
+        found = dict()
+
+        def report_error(lines, message):
+            for line in lines:
+                print(line.rstrip())
+            raise Exception(message)
+
+        confpath = Path(f'{self._repo.path}')
+        # This might be a worktree.  Make sure we have a config file and if not,
+        # walk up until se wee one.
+        while not os.path.exists(confpath / 'config'):
+            newpath = confpath.parent
+            if newpath == confpath:
+                raise Exception(f'Cannot find git config file in {self._repo.path}')
+            confpath = newpath
+
+        with open(confpath / 'config') as conffile:
+            lines = []
+            secpath = ''
+            section = None
+            for line in conffile:
+                lines.append(line)
+                # Match a section header.
+                match = re.match(r'^\[([^\s]*)( "([^"]*)")?\]$',
+                                 line.strip())
+                if match:
+                    prefix = match.group(1)
+                    suffix = match.group(3)  # Could be None
+                    secpath = f'{prefix}.{suffix}' if suffix else f'{prefix}'
+                    if not secpath in found:
+                        found[secpath] = set()
+                    section = found[secpath]
+                else:
+                    # Not a section header, try to match key = value.
+                    match = re.match(r'^\s*([^\s=]+) = (.+)$', line.strip())
+                    if match:
+                        matched_key = match.group(1)
+                        matched_value = match.group(2)
+                        item = f'{matched_key} = {matched_value}'
+                        if section is None:
+                            report_error(lines,
+                                         f'git config {item} not in section')
+
+                        if item in section:
+                            report_error(lines,
+                                         f'git config duplicate entry {secpath}.{item}')
+                        section.add(item)
